package client;

import java.net.*;

/**
 * Use the MessageSender to send text messages to other users.
 * The udp socket is used to send packets directly to another
 * user's computer, using the information (ip:port) received
 * from the server.
 *
 * Implement the marked part of the sendMessage()-methode below.
 * Take a look at the whole source code, especially at InstantMessenger
 * and NetworkUtilities. Use InstantMessenger.IM to have access to the
 * only existing instance of the InstantMessenger and the gui.
 */

public class MessageSender {
	/** The UDP socket with which we send. */
	private DatagramSocket socket;

	public MessageSender(DatagramSocket socket) {
		this.socket = socket;
	}

	/**
	 * Sends the message to the specified user and returns success or not.
	 * Send the packet in the specified format: type MESSAGE (1), messageLength (2),
	 * messageText (var).
	 */
	public boolean sendMessage(String messageText, User user) {
		try {
			/* Create packet (containing the messageText) and send it to the user */

			// Get some necessary information...
			int messageLength = messageText.length();
			byte[] message = new byte[3 + messageLength];
			int port = user.getPort();
			InetAddress ip = null;
			try {
				ip = InetAddress.getByName(user.showIP());
			} catch (UnknownHostException uhe) {
				System.out.println("Oops, unknown address. Cannot send: " + user.showIP());
				return false;
			}

			// Create the message (as byte array)
			message[0] = NetworkUtilities.MESSAGE;
			NetworkUtilities.put16(messageLength, message, 1);
			System.arraycopy(messageText.getBytes(), 0, message, 3, messageLength);

			// Create and send the UDP packet
			DatagramPacket packet = new DatagramPacket(message, message.length, ip, port);
			socket.send(packet);
		} catch (Exception e) {
			System.out.println("Oops, we caused an Error");
			e.printStackTrace();
			return false;
		}
		return true;
	}
}